/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file EnqueueBroadcastBuffer.h
 *
 * @date 2012-07-02
 * @author Philipp Kegel
 */

#ifndef ENQUEUEBROADCASTBUFFER_H_
#define ENQUEUEBROADCASTBUFFER_H_

#include "Request.h"

#include <dcl/DCLTypes.h>

#include <gcf/GCFTypedefs.h>

#include <cstddef>
#include <vector>

namespace dclgcf {

namespace message {

class EnqueueBroadcastBuffer : public Request {
public:
	EnqueueBroadcastBuffer(
			const std::vector<dcl::object_id>&  commandQueueIds,
			dcl::object_id                      commandId,
			dcl::object_id                      srcBufferId,
			const std::vector<dcl::object_id>&  dstBufferIds,
			size_t                              srcOffset,
			const std::vector<size_t>&          dstOffsets,
			size_t                              cb,
			const std::vector<dcl::object_id> * eventIdWaitList = nullptr,
			bool                                event = false);
	EnqueueBroadcastBuffer(
	        const EnqueueBroadcastBuffer& rhs);

	const std::vector<dcl::object_id>& commandQueueIds() const;
    dcl::object_id commandId() const;
    dcl::object_id srcBufferId() const;
    const std::vector<dcl::object_id>& dstBufferIds() const;
    size_t srcOffset() const;
    void dstOffsets(
    		std::vector<size_t>& dstOffsets) const;
    size_t cb() const;
    const std::vector<dcl::object_id>& eventIdWaitList() const;
    bool event() const;

    DECLARE_SERIALIZABLE_PUBLIC(EnqueueBroadcastBuffer, 100 + ENQUEUE_BROADCAST_BUFFER)

private:
    std::vector<dcl::object_id> _commandQueueIds;
    dcl::object_id _commandId;
    dcl::object_id _srcBufferId;
    std::vector<dcl::object_id> _dstBufferIds;
    gcf_uint32 _srcOffset;
    std::vector<gcf_uint32> _dstOffsets;
    gcf_uint32 _cb;
    std::vector<dcl::object_id> _eventIdWaitList;
    gcf_bool _event;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(EnqueueBroadcastBuffer)
};

} /* namespace message */

} /* namespace dclgcf */

#endif /* ENQUEUEBROADCASTBUFFER_H_ */
