/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/*!
 * \file InfoResponse.h
 *
 * \date 2011-05-19
 * \author Marius Holzwarth
 * \author Philipp Kegel
 */

#ifndef INFORESPONSE_H_
#define INFORESPONSE_H_

#include "Response.h"

#include <dcl/Binary.h>

#include <gcf/Serializable.h>

#include <cstddef>

namespace dclgcf {

namespace message {

/* forward declarations */
class Request;

/******************************************************************************/

/*!
 * \brief A response message containing a single piece of information on any type of OpenCL object.
 */
class InfoResponse: public Response {
public:
	InfoResponse(
			const Request& request,
			size_t         size,
			const void *   value);
	InfoResponse(
	        const InfoResponse& rhs);
	virtual ~InfoResponse();

    dcl::Binary param() const;

	DECLARE_SERIALIZABLE_PUBLIC(InfoResponse, 298)

private:
    dcl::Binary _param;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(InfoResponse)
};

} /* namespace message */

} /* namespace dclgcf */

#endif /* INFORESPONSE_H_ */
