/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file CLResponseDispatcher.h
 *
 * @date 2012-07-14
 * @author Philipp Kegel
 */

#ifndef CLRESPONSEDISPATCHER_H_
#define CLRESPONSEDISPATCHER_H_

#include <gcf/CallbackPolicy.h>
#include <gcf/GCFTypedefs.h>
#include <gcf/MessageListener.h>
#include <gcf/Process.h>
#include <gcf/Serializable.h>

#include <vector>

namespace dclgcf {

/* forward declarations */
class CommunicationManagerImpl;

/******************************************************************************/

namespace comm {

/**
 * @brief A central dispatch for incoming command response messages from GCF.
 *
 * Responses are distributed to the response buffers of their respective senders.
 */
class CLResponseDispatcher: public gcf::MessageListener {
public:
    /**
     * @brief Returns a GCF callback policy comprising the command response types processed by a response dispatcher.
     *
     * @return a callback policy
     */
    static const gcf::CallbackPolicy& callbackPolicy();


    CLResponseDispatcher(
            const CommunicationManagerImpl& connectionManager);
    virtual ~CLResponseDispatcher();

    /*
     * GCF message listener API
     */
    void dispatch(
    		gcf::Serializable&  message,
    		const gcf::Process& process);

private:
    /**
     * @brief A list of dOpenCL command response message types.
     *
     * This class is used internally for static initialization of a GCF callback
     * policy.
     */
    class MessageTypes : public std::vector<GCFTypeID_t> {
    public:
        MessageTypes();
    };


    const CommunicationManagerImpl& _communicationManager;
};

} /* namespace comm */

} /* namespace dclgcf */

#endif /* CLRESPONSEDISPATCHER_H_ */
