/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file DataDispatcher.h
 *
 * @date 2011-02-16
 * @author Philipp Kegel
 */

#ifndef DATADISPATCHER_H_
#define DATADISPATCHER_H_

#include <dcl/BlockingQueue.h>

#include <gcf/transport/IODataStream.h>

#include <cstddef>
#include <memory>
#include <thread>

namespace dclgcf {

namespace comm {

/* forward declarations */
class DataTransferImpl;

/******************************************************************************/

/**
 * This class actively monitors pending data transfers (one-by-one) in order to
 * execute callbacks that have been registered for these data transfers.
 * Callbacks are not supported by GCF data transfers; otherwise this class may
 * become obsolete.
 */
class DataDispatcher {
public:
	DataDispatcher();
	virtual ~DataDispatcher();

    void run();

	/**
	 * @brief Enqueue a data receipt.
	 *
	 * @param[in]  dataStream   the data stream to receive from
	 * @param[in]  size         the number of byte to receive
	 * @param[out] ptr          receive buffer
	 */
	std::shared_ptr<DataTransferImpl> receiveData(
			gcf::IODataStream&  dataStream,
			size_t              size,
			void *              ptr);

	/**
	 * @brief Enqueue a data sending.
	 *
	 * @param[in]  dataStream   the data stream to send through
	 * @param[in]  size         the number of bytes to send
	 * @param[in]  ptr          send buffer
	 */
	std::shared_ptr<DataTransferImpl> sendData(
			gcf::IODataStream&  dataStream,
			size_t              size,
			const void *        ptr);

private:
	dcl::BlockingQueue<std::shared_ptr<DataTransferImpl>> _dataTransfers;

	std::thread _worker;
};

} /* namespace comm */

} /* namespace dclgcf */

#endif /* DATADISPATCHER_H_ */
