/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file DataTransferImpl.h
 *
 * @date 2011-12-17
 * @author Philipp Kegel
 */

#ifndef DATATRANSFERIMPL_H_
#define DATATRANSFERIMPL_H_

#include <dcl/DataTransfer.h>

#include <gcf/transport/IncomingDataStream.h>
#include <gcf/transport/OutgoingDataStream.h>

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <boost/shared_ptr.hpp>

#include <cstddef>
#include <functional>
#include <mutex>
#include <vector>

namespace dclgcf {

namespace comm {

class DataTransferImpl :
	virtual public dcl::DataTransfer {
public:
    void setCallback(
            const std::function<void (cl_int)>& notify);

    void abort();

    virtual void execute() = 0;

protected:
	virtual cl_int status() const = 0;
	void triggerCallbacks();

	std::vector<std::function<void (cl_int)>> _callbacks;
	std::mutex _callbacksMutex;
};

/******************************************************************************/

/**
 * @brief A handle for an asynchronous data receipt.
 */
class DataReceipt: public DataTransferImpl {
public:
	DataReceipt(
			const boost::shared_ptr<const gcf::IncomingDataStream::Status>& status);

	unsigned long submit() const;
	unsigned long start() const;
	unsigned long end() const;

	bool isComplete() const;
	void wait() const;

	void execute();

private:
	cl_int status() const;

	boost::shared_ptr<const gcf::IncomingDataStream::Status> _status;
};

/******************************************************************************/

/**
 * @brief A handle for an asynchronous data sending.
 */
class DataSending: public DataTransferImpl {
public:
	DataSending(
			const boost::shared_ptr<const gcf::OutgoingDataStream::Status>& status);

	unsigned long submit() const;
	unsigned long start() const;
	unsigned long end() const;

	bool isComplete() const;
	void wait() const;

	void execute();

private:
	cl_int status() const;

	boost::shared_ptr<const gcf::OutgoingDataStream::Status> _status;
};

} /* namespace comm */

} /* namespace dclgcf */

#endif /* DATATRANSFERIMPL_H_ */
