/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file DeviceIDsResponse.cpp
 *
 * @date 2011-05-18
 * @author Karsten Jeschkies
 */

#include "DeviceIDsResponse.h"

#include <dclgcf/message/Response.h>

#include <dcl/DCLTypes.h>

#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#include <gcf/serialization/ByteBuffer.h>

#include <cassert>
#include <cstddef>
#include <vector>

namespace gcf {

//Add support to serialize device_ids vector
template<>
size_t writeCustomSTR<dclgcf::message::DeviceIDsResponse> (
		dclgcf::message::DeviceIDsResponse *object,
		const gcf::GenericSerializeTemplateRecord& gstr,
		char *buffer) {
	assert(gstr.getType() == gcf::ST_Custom1);

	gcf::ByteBuffer buf(buffer, gstr.getLength());
	std::vector<dcl::object_id>& deviceIds =
			*gcf::GenericSerializeTemplateRecord::getAttribute<
					dclgcf::message::DeviceIDsResponse, std::vector<dcl::object_id>,
					gcf::ST_Custom1>(object, gstr);

	buf << deviceIds;

	return buf.getLength();
}

template<>
size_t readCustomSTR<dclgcf::message::DeviceIDsResponse> (
		dclgcf::message::DeviceIDsResponse *object,
		const gcf::GenericSerializeTemplateRecord& gstr,
		const char *buffer) {
	assert(gstr.getType() == gcf::ST_Custom1);

	gcf::ByteBuffer buf(buffer, gstr.getLength());
	std::vector<dcl::object_id>& deviceIds =
			*gcf::GenericSerializeTemplateRecord::getAttribute<
					dclgcf::message::DeviceIDsResponse, std::vector<dcl::object_id>,
					gcf::ST_Custom1>(object, gstr);

	deviceIds.clear();
	buf >> deviceIds;

	return buf.getLength();
}

} /* namespace gcf */

/******************************************************************************/

namespace dclgcf {

namespace message {

/* forward declarations */
class Request;

/******************************************************************************/

DeviceIDsResponse::DeviceIDsResponse() {
	initSerializable();
}

DeviceIDsResponse::DeviceIDsResponse(
		const Request& request,
		const std::vector<dcl::object_id>& deviceIds) :
	Response(request), _deviceIds(deviceIds) {
	initSerializable();
}

DeviceIDsResponse::DeviceIDsResponse(
		const DeviceIDsResponse& rhs) :
	Response(rhs), _deviceIds(rhs._deviceIds) {
	initSerializable();
}

DeviceIDsResponse::~DeviceIDsResponse() { }

const std::vector<dcl::object_id>& DeviceIDsResponse::deviceIds(void) const {
	return _deviceIds;
}

} /* namespace message */

} /* namespace dclgcf */

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
        dclgcf::message::DeviceIDsResponse,
        dclgcf::message::Response,
        ADD_ATTRIBUTE_CUSTOM(dclgcf::message::DeviceIDsResponse, _deviceIds, gcf::ST_Custom1,
                std::vector<dcl::object_id>, 256, gcf::Reliable, gcf::Public);
        )
