/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/*!
 * \file    DeviceInfosResponse.h
 *
 * \date    2011-05-19
 * \author  Marius Holzwarth
 * \author  Philipp Kegel
 */

#ifndef DEVICEINFOSRESPONSE_H_
#define DEVICEINFOSRESPONSE_H_

#include <dclgcf/message/Response.h>
#include <dclgcf/message/Request.h>

#include <dcl/Binary.h>

#include <gcf/Serializable.h>

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <cstddef>
#include <map>

namespace dclgcf {

namespace message {

/*!
 * \brief A response message containing a set of device information.
 *
 * Unlike InfoResponse, this class is designed to provide a set of information
 * items rather than a single item.
 *
 * !!! This message class is currently not used. !!!
 */
class DeviceInfosResponse: public Response {
public:
    DeviceInfosResponse(
            const Request&                                      request,
            const std::map<cl_device_info, const dcl::Binary>&  params);
    DeviceInfosResponse(
            const Request&  request,
            cl_device_info  paramName,
            size_t          size,
            const void *    value);
	DeviceInfosResponse(
	        const DeviceInfosResponse& rhs);
	virtual ~DeviceInfosResponse();

	dcl::Binary param(
	        cl_device_info paramName) const;
	std::map<cl_device_info, const dcl::Binary> params() const;

	DECLARE_SERIALIZABLE_PUBLIC(DeviceInfosResponse, 200 + Request::GET_DEVICE_INFO)

private:
    std::map<cl_device_info, const dcl::Binary> _params;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(DeviceInfosResponse)
};

} /* namespace message */

} /* namespace dclgcf */

#endif /* DEVICEINFOSRESPONSE_H_ */
