/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file ErrorResponse.cpp
 *
 * @date 2011-05-12
 * @author Karsten Jeschkies
 */

#include <dclgcf/message/ErrorResponse.h>
#include <dclgcf/message/Response.h>

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

namespace dclgcf {

namespace message {

/* forward declarations */
class Request;

/******************************************************************************/

ErrorResponse::ErrorResponse() :
	_errcode(CL_SUCCESS) {
	initSerializable();
}

ErrorResponse::ErrorResponse(
		const Request& request,
		cl_int errcode) :
	Response(request), _errcode(static_cast<gcf_int32>(errcode)) {

	initSerializable();
}

ErrorResponse::ErrorResponse(
		const ErrorResponse& rhs) :
	Response(rhs), _errcode(rhs._errcode) {
	initSerializable();
}

ErrorResponse::~ErrorResponse() { }

cl_int ErrorResponse::errcode() const {
	return static_cast<cl_int>(_errcode);
}

} /* namespace message */

} /* namespace dclgcf */

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
        dclgcf::message::ErrorResponse,
        dclgcf::message::Response,
        ADD_ATTRIBUTE_DEFAULT(dclgcf::message::ErrorResponse, _errcode);
        )
