/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file GetKernelInfo.cpp
 *
 * @date 2011-06-22
 * @author Karsten Jeschkies
 * @author Philipp Kegel
 */

#include <dclgcf/message/GetKernelInfo.h>
#include <dclgcf/message/Request.h>

#include <dcl/DCLTypes.h>

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

/******************************************************************************
 * Get kernel info
 ******************************************************************************/

namespace dclgcf {

namespace message {

GetKernelInfo::GetKernelInfo() {
    initSerializable();
}

GetKernelInfo::GetKernelInfo(
        dcl::object_id kernelId,
        cl_kernel_info paramName) :
        _kernelId(kernelId), _paramName(static_cast<gcf_uint32>(paramName)) {
    initSerializable();
}

GetKernelInfo::GetKernelInfo(
        const GetKernelInfo& rhs) :
        Request(rhs), _kernelId(rhs._kernelId), _paramName(rhs._paramName) {
    initSerializable();
}

GetKernelInfo::~GetKernelInfo() { }

dcl::object_id GetKernelInfo::kernelId() const {
    return _kernelId;
}

cl_kernel_info GetKernelInfo::paramName() const {
    return static_cast<cl_kernel_info>(_paramName);
}

} /* namespace message */

} /* namespace dclgcf */

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
        dclgcf::message::GetKernelInfo,
        dclgcf::message::Request,
        ADD_ATTRIBUTE_DEFAULT(dclgcf::message::GetKernelInfo, _kernelId);
        ADD_ATTRIBUTE_DEFAULT(dclgcf::message::GetKernelInfo, _paramName);
        )

/******************************************************************************
 * Get kernel work group info
 ******************************************************************************/

namespace dclgcf {

namespace message {

GetKernelWorkGroupInfo::GetKernelWorkGroupInfo() {
    initSerializable();
}

GetKernelWorkGroupInfo::GetKernelWorkGroupInfo(
        dcl::object_id kernelId,
        dcl::object_id deviceId,
        cl_kernel_work_group_info paramName) :
        _kernelId(kernelId), _deviceId(deviceId),
        _paramName(static_cast<gcf_uint32>(paramName)) {
    initSerializable();
}

GetKernelWorkGroupInfo::GetKernelWorkGroupInfo(
        const GetKernelWorkGroupInfo& rhs) :
        Request(rhs), _kernelId(rhs._kernelId), _deviceId(rhs._deviceId),
        _paramName(rhs._paramName) {
    initSerializable();
}

GetKernelWorkGroupInfo::~GetKernelWorkGroupInfo() { }

dcl::object_id GetKernelWorkGroupInfo::kernelId() const {
    return _kernelId;
}

dcl::object_id GetKernelWorkGroupInfo::deviceId() const {
    return _deviceId;
}

cl_kernel_info GetKernelWorkGroupInfo::paramName() const {
    return static_cast<cl_kernel_work_group_info>(_paramName);
}

} /* namespace message */

} /* namespace dclgcf */

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
        dclgcf::message::GetKernelWorkGroupInfo,
        dclgcf::message::Request,
        ADD_ATTRIBUTE_DEFAULT(dclgcf::message::GetKernelWorkGroupInfo, _kernelId);
        ADD_ATTRIBUTE_DEFAULT(dclgcf::message::GetKernelWorkGroupInfo, _deviceId);
        ADD_ATTRIBUTE_DEFAULT(dclgcf::message::GetKernelWorkGroupInfo, _paramName);
        )
