/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/*!
 * \file Request.cpp
 *
 * \date 2011-04-06
 * \author Philipp Kegel
 */

#include <dclgcf/message/Request.h>

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#ifdef SERIALIZE_BY_HAND
#include <gcf/serialization/ByteBuffer.h>

#include <cassert>
#endif

namespace dclgcf {

namespace message {

gcf_uint32 Request::requestCount = 0;

Request::Request() : id(++requestCount) {
	initSerializable();
}

Request::Request(const Request& rhs) :
	id(rhs.id) {
	initSerializable();
}

Request::~Request() { }

#ifndef SERIALIZE_BY_HAND
} /* namespace message */

} /* namespace dclgcf */

IMPLEMENT_SERIALIZABLE_EVENT(
		dclgcf::message::Request,
		ADD_ATTRIBUTE_DEFAULT(dclgcf::message::Request, id);
)
#else
gcf::TransmissionPolicy Request::serialize(
		void *buf, gcf_size_t& length,
		const gcf::SerializationPolicy& policy) {
	gcf::ByteBuffer buffer(buf, length);

	buffer << _command << _id;
	buffer.flip();
	length = buffer.getLength();

	return gcf::TransmissionPolicy();
}

void Request::deserialize(
		const void* buf, gcf_size_t &length,
		gcf::SerializationModule &) {
	gcf::ByteBuffer buffer(buf, length);

	buffer >> _command;
	buffer >> _id;
	buffer.flip();
	length = buffer.getLength();
}

void Request::updateMask() {
	return;
}

void Request::resetMask() {
	return;
}

void Request::initSerializable() {
	return;
}

bool Request::hasChanged(
		bool forceUpdate) {
	return true;
}

gcf::SerializeTemplate<Request> *Request::createTemplate() {
	static gcf::SerializeTemplate<Request> templ;
	static bool initialized;

	if (!initialized) {
		initialized = true;
	}

	return &templ;
}

} /* namespace message */

} /* namespace dclgcf */

IMPLEMENT_REGISTER_SERIALIZABLE(dclgcf::message::Request)
DEFINE_SERIALIZE_TEMPLATE_MASK(dclgcf::message::Request)
DEFINE_SERIALIZE_TEMPLATE(dclgcf::message::Request, serTempl)
#endif
