/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file utility.cpp
 *
 * @date 2013-10-29
 * @author Philipp Kegel
 */

#include <gcf/GCFTypedefs.h>

#include <cstddef>
#include <iterator>
#include <vector>

namespace dclgcf {

namespace message {

void workDimensionHtoN(
        const std::vector<size_t>& in,
        std::vector<gcf_uint32>& out) {
    std::vector<size_t>::const_iterator src;
    std::vector<gcf_uint32>::iterator dst;

    out.resize(in.size());
    src = std::begin(in);
    dst = std::begin(out);

    while (src != std::end(in)) {
        *dst++ = static_cast<gcf_uint32>(*src++);
    }
}

void workDimensionNtoH(
        const std::vector<gcf_uint32>& in,
        std::vector<size_t>& out) {
    std::vector<gcf_uint32>::const_iterator src;
    std::vector<size_t>::iterator dst;

    out.resize(in.size());
    src = std::begin(in);
    dst = std::begin(out);

    while (src != std::end(in)) {
        *dst++ = static_cast<size_t>(*src++);
    }
}

} /* namespace message */

} /* namespace dclgcf */
